
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "myType.h"
#include "com.h"

#define R_FLAG  O_NOCTTY|O_NDELAY|O_RDONLY
#define W_FLAG  O_NOCTTY|O_NDELAY|O_WRONLY
#define RW_FLAG O_NOCTTY|O_NDELAY|O_RDWR


int COM_open(BYTE port, BYTE mode, WORD speed, char parity, BYTE bits, BYTE stop, BYTE flow)
{
	int PortId;
	struct termios SerialFlags;
	char dev[] = "/dev/ttyS?";
	
	if (port > 9)
		return (-1);
	
	dev[9] = (port | 0x30); // num. -> car.
		
	switch (mode)
	{
		case READ:   		PortId = open(dev, R_FLAG);
					break;
		case WRITE:   		PortId = open(dev, W_FLAG);
					break;
		case READ_WRITE:	PortId = open(dev, RW_FLAG);
					break;
		default:  		return -1;
	}

	if (PortId < 0)
	{
		close(PortId);
		return -1;
	}
	
	if (tcgetattr(PortId,&SerialFlags) < 0)
	{
		close(PortId);
		return -1;
	}
   
	SerialFlags.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	SerialFlags.c_lflag |= NOFLSH;
	SerialFlags.c_iflag &= ~(IXON | IXOFF | IXANY);
	SerialFlags.c_oflag &= ~OPOST;
	SerialFlags.c_cflag |= (CREAD|CLOCAL);
	
	SerialFlags.c_cc[VTIME] = 0;
	SerialFlags.c_cc[VMIN] = 1;
		
	switch (speed)
	{
		case 1200:  cfsetispeed(&SerialFlags,B1200);
								cfsetospeed(&SerialFlags,B1200);
								break;
		case 2400:  cfsetispeed(&SerialFlags,B2400);
								cfsetospeed(&SerialFlags,B2400);
								break;
		case 4800:  cfsetispeed(&SerialFlags,B4800);
								cfsetospeed(&SerialFlags,B4800);
								break;
		case 9600:  cfsetispeed(&SerialFlags,B9600);
								cfsetospeed(&SerialFlags,B9600);
								break;
		case 19200: cfsetispeed(&SerialFlags,B19200);
								cfsetospeed(&SerialFlags,B19200);
								break;
		case 38400: cfsetispeed(&SerialFlags,B38400);
								cfsetospeed(&SerialFlags,B38400);
								break;
		case 57600: cfsetispeed(&SerialFlags,B57600);
								cfsetospeed(&SerialFlags,B57600);
								break;
		default:    close(PortId);
								return -2;
	}
  
	SerialFlags.c_cflag &= ~(PARENB | PARODD);
	switch (parity)
	{
		case 'n': break;
		case 'o': SerialFlags.c_cflag |= (PARENB|PARODD);
							SerialFlags.c_iflag |= (INPCK | ISTRIP); // ?
							break;
		case 'e': SerialFlags.c_cflag |= PARENB;
							SerialFlags.c_iflag |= (INPCK | ISTRIP); // ?
							break;
		default:  close(PortId);
							return -3;
	}
       
	SerialFlags.c_cflag &= ~CSIZE;
	switch (bits)
	{
		case 7:  SerialFlags.c_cflag |= CS7;
						 break;
		case 8:  SerialFlags.c_cflag |= CS8;
						 break;
		default: close(PortId);
						 return -4;
	}
        
	SerialFlags.c_cflag &= ~CSTOPB;
	switch (stop)
	{
		case 1:  break;
		case 2:  SerialFlags.c_cflag |= CSTOPB;
						 break;
		default: close(PortId);
						 return -5;
	}
 
	SerialFlags.c_cflag &= ~CRTSCTS;
	switch (flow)
	{
		case NO_HANDSHAKE:	break;
		case HW_HANDSHAKE:  	SerialFlags.c_cflag |= CRTSCTS;
										break;
		default: 	      close(PortId);
							      return -6;
	}
 
	if (tcsetattr(PortId,TCSANOW,&SerialFlags) < 0)
	{
		close(PortId);
		return -6;
	}
 
	return PortId;
} 

void COM_close(int com)
{
	close(com);
	return;
}  


int COM_write (int com, char buf[], int n)
{
	return write(com, buf, n);
}

int COM_read (int com, char* buf, int n)
{
	return read(com, buf, n);
}
